--------------------------------------------
-- TABLETOP ASSISTANT (ALPHA BUILD)		  --
-- CREATED WITH LUA FOR THE 'LOVE' ENGINE --
-- BY ENSAYIA (ENSAYIA@GMAIL.COM)		  --
--------------------------------------------
-- SPECIAL THANKS TO TAEHL FOR BUTTON 	  --
-- GRAPHICS AND DEBUGGING HELP			  --
--------------------------------------------

function love.load()
	load_buttons()		-- I set a table inside a function and placed it at EOF just to be neater.
	roll = {0,0,0,0,0}		-- Define a table for saving user's roll values.
	rollvar = 0		-- Buffer to hold the current roll value.
	i = 0		-- Increment Counter.
	msg = "No Current Roll"		-- Placeholder message set before you roll any dice.		
	math.randomseed=(os.time())		-- Grab the time from the OS as a nice random seed.
	math.random(0,1)		-- A friend suggested adding this helps ballance out the random seed.
	
	buttonblip = love.audio.newSource("audio/buttonblip.ogg", static)		-- Sound used when the button is clicked.
end

function love.draw()
	for k, thing in pairs(button) do		-- Using a for loop to place all my buttons on the screen automatically.
		love.graphics.draw(thing.image,thing.x,thing.y,0,1,1,1,1)
	end
	
	love.graphics.rectangle("line", 0, 0, 200, 80)		-- A few drawn rectangles, for neatness sake.
	love.graphics.rectangle("line", 0, 81, 200, 40)
	
	love.graphics.print("Saved Rolls",10,15)			-- Saved Rolls chart is drawn on screen and set to.
	love.graphics.print("",10,25)						-- the defualt values.
	love.graphics.print(roll[1],10,35)
	love.graphics.print(roll[2],10,45)
	love.graphics.print(roll[3],10,55)
	love.graphics.print(roll[4],10,65)
	love.graphics.print(roll[5],10,75)
	
	love.graphics.print(msg,10,95)		-- Using msg to display what the user rolled.
	love.graphics.print("",10,105)
	love.graphics.print(rollvar,10,115)		-- Display whatever the current roll is.
end


function love.update(dt)		-- DT is time since last frame in seconds. It's usefull for things like calculations with timing.
	bclick = love.mouse.isDown("l")		--Check if we've clicked the left mouse.
	mousex = love.mouse.getX()		-- Store the mouse cursor's current x coordinate.
	mousey = love.mouse.getY()		-- Store the mouse cursor's current y coordinate.
	
	for k, b in pairs(button) do		-- Check the values of each table in the buttons table defined at EOF.
		if bclick and not notbclick and (mousex > b.x and mousex < b.x + 128) and (mousey < b.y + 32 and mousey > b.y) then
		-- (ABOVE) Check each button's coordiates againt mouse coordinates to see which one we are clicking on.	
			if b.code then b.code()		-- Special buttons have their own code defined in their table. This checks if there is such code
			else						-- and executes it or otherwise moves on and assumes we have a dice roll button.
				love.audio.stop(buttonblip)		-- Stopping the blip sound before we play it prevents odd sound errors,
				love.audio.play(buttonblip)		-- Play a sound.
				rollvar = math.random(1,b.toproll)	-- Grab a random number between 1 and whatever the max roll is for that die defined in it's table.
				msg = "Rolled 1".. k		--  Reference the table's name to derive which die we have rolled and output for the user to see.
			end
		end
	end
	
	notbclick = bclick		-- Used in the button code for checking if the button is already pressed or not. Prevents accidental
							-- rapid-fire rolling by forcing the program to release the button before it registers the next click.
							-- I must do this because LOVE can only detect if a button is held and not pressed once... /facepalm .
end

function load_buttons()		-- I placed all the button tables and specialized code in their own function at the EOF to make things
	button =				-- neater and not all crammed at the beginning, or in love.load().
		{
			d4 = 
				{
					image = love.graphics.newImage("image/b_d4.png"),
					x = 10,
					y = 150,
					toproll = 4		-- Determines the maximum roll for the dice type we're using.
				},
				
			d6 = 
				{
					image = love.graphics.newImage("image/b_d6.png"),
					x = 10,
					y = 183,
					toproll = 6		-- Determines the maximum roll for the dice type we're using.
				},
				
			d8 = 
				{
					image = love.graphics.newImage("image/b_d8.png"),
					x = 10,
					y = 216,
					toproll = 8		-- Determines the maximum roll for the dice type we're using.
				},
				
			d10 = 
				{
					image = love.graphics.newImage("image/b_d10.png"),
					x = 140,
					y = 150,
					toproll = 10		-- Determines the maximum roll for the dice type we're using.
				},
				
			d12 = 
				{
					image = love.graphics.newImage("image/b_d12.png"),
					x = 140,
					y = 183,
					toproll = 12		-- Determines the maximum roll for the dice type we're using.
				},
				
			d20 = 
				{
					image = love.graphics.newImage("image/b_d20.png"),
					x = 140,
					y = 216,
					toproll = 20		-- Determines the maximum roll for the dice type we're using.
				},
				
			d100 = 
				{
					image = love.graphics.newImage("image/b_d100.png"),
					x = 270,
					y = 150,
					toproll = 100		-- Determines the maximum roll for the dice type we're using.
				},
			saveroll =
				{
					image = love.graphics.newImage("image/b_saveroll.png"),
					x = 270,
					y = 183,
					code = function()
	
							if i <= 5 then i = i + 1 end		-- The 'tape' function here allows the user to store up to 5 of rolls by
																-- when pressing the save roll button. After the user saves 5 it starts 
								if i > 5 then 					-- bumping list up vertically and storing the latest roll at the bottom.
								for i2 = 1,5,1 do				-- Using an increment and for loop, we shift all the values by one position.
									roll[i2] = roll[i2 + 1]
								end
							end
			
							if i > 5 then i = 5 end				-- Currently there's a maximum display of 5 saved rolls, we can technically
																-- store as many as we want but since we limit what we display, the user can't
																-- see them so there's no point.
							
							roll[i] = rollvar					-- Sets what's in the roll buffer to the appropreate storage spot before the 
																-- function ends.
					end
				},
		}
end